@@ -1,18 +1,10 @@
 #!/bin/sh
 #
-# Copyright (C) 2010, 2012, 2014, 2016, 2017  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2010, 2012, 2014, 2016  Internet Systems Consortium, Inc. ("ISC")
 #
-# Permission to use, copy, modify, and/or distribute this software for any
-# purpose with or without fee is hereby granted, provided that the above
-# copyright notice and this permission notice appear in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-# AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-# OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-# PERFORMANCE OF THIS SOFTWARE.
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 # $Id: tests.sh,v 1.3 2010/06/08 23:50:24 tbox Exp $
 
@@ -24,19 +16,13 @@ DIGOPTS="+tcp +noadd +nosea +nostat +nocmd +dnssec -p 5300"
 status=0
 ret=0
 
-algs=""
-have_rsa=`grep rsa supported`
-if [ "x$have_rsa" != "x" ]; then
-    algs="rsa "
-fi
-have_ecc=`grep ecc supported`
-if [ "x$have_ecc" != "x" ]; then
-    algs=$algs"ecc "
-fi
-have_ecx=`grep ecc supported`
-if [ "x$have_ecx" != "x" ]; then
-    algs=$algs"ecx "
-fi
+supported=`cat supported`
+case $supported in
+    rsaonly) algs="rsa" ;;
+    ecconly) algs="ecc" ;;
+    both) algs="rsa ecc" ;;
+esac
+
 
 for alg in $algs; do
     zonefile=ns1/$alg.example.db 
@@ -51,12 +37,7 @@ for alg in $algs; do
 
     echo "I:testing inline signing with PKCS#11 keys ($alg)"
 
-    $DIG $DIGOPTS ns.$alg.example. @10.53.0.1 a > dig.out.$alg.0 || ret=1
-    if [ $ret != 0 ]; then echo "I:failed"; fi
-    status=`expr $status + $ret`
-    count0=`grep RRSIG dig.out.$alg.0 | wc -l`
-
-    $NSUPDATE -v > upd.log.$alg <<END || status=1
+    $NSUPDATE > /dev/null <<END || status=1
 server 10.53.0.1 5300
 ttl 300
 zone $alg.example.
@@ -67,11 +48,11 @@ END
     echo "I:waiting 20 seconds for key changes to take effect"
     sleep 20
 
-    $DIG $DIGOPTS ns.$alg.example. @10.53.0.1 a > dig.out.$alg || ret=1
+    $DIG $DIGOPTS ns.$alg.example. @10.53.0.1 a > dig.out || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
-    count=`grep RRSIG dig.out.$alg | wc -l`
-    if [ $count -le $count0 ]; then echo "I:failed"; status=1; fi
+    count=`grep RRSIG dig.out | wc -l`
+    if [ $count != 4 ]; then echo "I:failed"; status=1; fi
 
     echo "I:testing PKCS#11 key destroy ($alg)"
     ret=0
@@ -80,7 +61,6 @@ END
     case $alg in
         rsa) id=02 ;;
         ecc) id=04 ;;
-	ecx) id=06 ;;
     esac
     $PK11DEL -i $id -w0 > /dev/null 2>&1 || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
