@@ -1,17 +1,18 @@
 /*
- * Copyright (C) 1998-2001, 2004, 2005, 2007, 2016, 2017  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1998-2001, 2004, 2005, 2007, 2016  Internet Systems Consortium, Inc. ("ISC")
  *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+/* $Id: thread.c,v 1.24 2007/06/19 23:47:19 tbox Exp $ */
+
 #include <config.h>
 
 #include <process.h>
 
 #include <isc/thread.h>
-#include <isc/util.h>
 
 isc_result_t
 isc_thread_create(isc_threadfunc_t start, isc_threadarg_t arg,
@@ -57,12 +58,6 @@ isc_thread_setconcurrency(unsigned int level) {
 	 */
 }
 
-void
-isc_thread_setname(isc_thread_t thread, const char *name) {
-	UNUSED(thread);
-	UNUSED(name);
-}
-
 void *
 isc_thread_key_getspecific(isc_thread_key_t key) {
 	return(TlsGetValue(key));
