@@ -1,17 +1,9 @@
 /*
- * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2000-2016  Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
 /*! \file */
@@ -119,9 +111,13 @@ LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_map = { "map", free_map };
 LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_list = { "list", free_list };
 LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_tuple = { "tuple", free_tuple };
 LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_sockaddr = { "sockaddr", free_noop };
-LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_netprefix = { "netprefix", free_noop };
+LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_netprefix =
+	{ "netprefix", free_noop };
 LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_void = { "void", free_noop };
-LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_fixedpoint = { "fixedpoint", free_noop };
+LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_fixedpoint =
+	{ "fixedpoint", free_noop };
+LIBISCCFG_EXTERNAL_DATA cfg_rep_t cfg_rep_percentage =
+	{ "percentage", free_noop };
 
 /*
  * Configuration type definitions.
@@ -153,24 +149,34 @@ cfg_print_chars(cfg_printer_t *pctx, const char *text, int len) {
 
 static void
 print_open(cfg_printer_t *pctx) {
-	cfg_print_chars(pctx, "{\n", 2);
-	pctx->indent++;
+	if ((pctx->flags & CFG_PRINTER_ONELINE) != 0)
+		cfg_print_cstr(pctx, "{ ");
+	else {
+		cfg_print_cstr(pctx, "{\n");
+		pctx->indent++;
+	}
 }
 
 static void
 print_indent(cfg_printer_t *pctx) {
 	int indent = pctx->indent;
+	if ((pctx->flags & CFG_PRINTER_ONELINE) != 0) {
+		cfg_print_cstr(pctx, " ");
+		return;
+	}
 	while (indent > 0) {
-		cfg_print_chars(pctx, "\t", 1);
+		cfg_print_cstr(pctx, "\t");
 		indent--;
 	}
 }
 
 static void
 print_close(cfg_printer_t *pctx) {
-	pctx->indent--;
-	print_indent(pctx);
-	cfg_print_chars(pctx, "}", 1);
+	if ((pctx->flags & CFG_PRINTER_ONELINE) == 0) {
+		pctx->indent--;
+		print_indent(pctx);
+	}
+	cfg_print_cstr(pctx, "}");
 }
 
 isc_result_t
@@ -301,7 +307,7 @@ cfg_print_tuple(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 
 void
 cfg_doc_tuple(cfg_printer_t *pctx, const cfg_type_t *type) {
-	const cfg_tuplefielddef_t *fields;
+	const cfg_tuplefielddef_t *fields = type->of;
 	const cfg_tuplefielddef_t *f;
 	isc_boolean_t need_space = ISC_FALSE;
 
@@ -459,6 +465,7 @@ cfg_parser_create(isc_mem_t *mctx, isc_log_t *lctx, cfg_parser_t **ret) {
 	pctx->callbackarg = NULL;
 	pctx->token.type = isc_tokentype_unknown;
 	pctx->flags = 0;
+	pctx->buf_name = NULL;
 
 	memset(specials, 0, sizeof(specials));
 	specials['{'] = 1;
@@ -525,6 +532,20 @@ cfg_parser_setcallback(cfg_parser_t *pctx,
 	pctx->callbackarg = arg;
 }
 
+void
+cfg_parser_reset(cfg_parser_t *pctx) {
+	REQUIRE(pctx != NULL);
+
+	if (pctx->lexer != NULL)
+		isc_lex_close(pctx->lexer);
+
+	pctx->seen_eof = ISC_FALSE;
+	pctx->ungotten = ISC_FALSE;
+	pctx->errors = 0;
+	pctx->warnings = 0;
+	pctx->line = 0;
+}
+
 /*
  * Parse a configuration using a pctx where a lexer has already
  * been set up with a source.
@@ -545,8 +566,7 @@ parse2(cfg_parser_t *pctx, const cfg_type_t *type, cfg_obj_t **ret) {
 
 	if (result != ISC_R_SUCCESS) {
 		/* Parsing failed but no errors have been logged. */
-		cfg_parser_error(pctx, 0, "parsing failed: %s",
-				 isc_result_totext(result));
+		cfg_parser_error(pctx, 0, "parsing failed");
 		goto cleanup;
 	}
 
@@ -565,6 +585,7 @@ cfg_parse_file(cfg_parser_t *pctx, const char *filename,
 	       const cfg_type_t *type, cfg_obj_t **ret)
 {
 	isc_result_t result;
+	cfg_listelt_t *elt;
 
 	REQUIRE(pctx != NULL);
 	REQUIRE(filename != NULL);
@@ -572,7 +593,15 @@ cfg_parse_file(cfg_parser_t *pctx, const char *filename,
 	REQUIRE(ret != NULL && *ret == NULL);
 
 	CHECK(parser_openfile(pctx, filename));
-	CHECK(parse2(pctx, type, ret));
+
+	result = parse2(pctx, type, ret);
+
+	/* Clean up the opened file */
+	elt = ISC_LIST_TAIL(pctx->open_files->value.list);
+	INSIST(elt != NULL);
+	ISC_LIST_UNLINK(pctx->open_files->value.list, elt, link);
+	ISC_LIST_APPEND(pctx->closed_files->value.list, elt, link);
+
  cleanup:
 	return (result);
 }
@@ -582,30 +611,39 @@ isc_result_t
 cfg_parse_buffer(cfg_parser_t *pctx, isc_buffer_t *buffer,
 	const cfg_type_t *type, cfg_obj_t **ret)
 {
-	return (cfg_parse_buffer4(pctx, buffer, NULL, 0, type, 0, ret));
+	return (cfg_parse_buffer3(pctx, buffer, NULL, 0, type, ret));
 }
 
 isc_result_t
-cfg_parse_buffer4(cfg_parser_t *pctx, isc_buffer_t *buffer,
-		  const char *file, unsigned int line,
-		  const cfg_type_t *type, unsigned int flags,
+cfg_parse_buffer2(cfg_parser_t *pctx, isc_buffer_t *buffer,
+		  const char *file, const cfg_type_t *type,
 		  cfg_obj_t **ret)
+{
+	return (cfg_parse_buffer3(pctx, buffer, file, 0, type, ret));
+}
+
+isc_result_t
+cfg_parse_buffer3(cfg_parser_t *pctx, isc_buffer_t *buffer,
+		  const char *file, unsigned int line,
+		  const cfg_type_t *type, cfg_obj_t **ret)
 {
 	isc_result_t result;
+
 	REQUIRE(pctx != NULL);
 	REQUIRE(type != NULL);
 	REQUIRE(buffer != NULL);
 	REQUIRE(ret != NULL && *ret == NULL);
-	REQUIRE((flags & ~(CFG_PCTX_NODEPRECATED)) == 0);
-
-	UNUSED(file);
-	UNUSED(line);
 
 	CHECK(isc_lex_openbuffer(pctx->lexer, buffer));
 
-	pctx->flags = flags;
+	pctx->buf_name = file;
+
+	if (line != 0U)
+		CHECK(isc_lex_setsourceline(pctx->lexer, line));
 
 	CHECK(parse2(pctx, type, ret));
+	pctx->buf_name = NULL;
+
  cleanup:
 	return (result);
 }
@@ -686,6 +724,75 @@ LIBISCCFG_EXTERNAL_DATA cfg_type_t cfg_type_void = {
 	"void", cfg_parse_void, cfg_print_void, cfg_doc_void, &cfg_rep_void,
 	NULL };
 
+/*
+ * percentage
+ */
+isc_result_t
+cfg_parse_percentage(cfg_parser_t *pctx, const cfg_type_t *type,
+		     cfg_obj_t **ret)
+{
+	char *endp;
+	isc_result_t result;
+	cfg_obj_t *obj = NULL;
+	isc_uint64_t percent;
+
+	REQUIRE(pctx != NULL);
+	REQUIRE(ret != NULL && *ret == NULL);
+
+	UNUSED(type);
+
+	CHECK(cfg_gettoken(pctx, 0));
+	if (pctx->token.type != isc_tokentype_string) {
+		cfg_parser_error(pctx, CFG_LOG_NEAR,
+				 "expected percentage");
+		return (ISC_R_UNEXPECTEDTOKEN);
+	}
+
+	percent = isc_string_touint64(TOKEN_STRING(pctx), &endp, 10);
+	if (*endp != '%' || *(endp+1) != 0) {
+		cfg_parser_error(pctx, CFG_LOG_NEAR,
+				 "expected percentage");
+		return (ISC_R_UNEXPECTEDTOKEN);
+	}
+
+	CHECK(cfg_create_obj(pctx, &cfg_type_percentage, &obj));
+	obj->value.uint32 = (isc_uint32_t)percent;
+	*ret = obj;
+
+ cleanup:
+	return (result);
+}
+
+void
+cfg_print_percentage(cfg_printer_t *pctx, const cfg_obj_t *obj) {
+	char buf[64];
+	int n;
+
+	REQUIRE(pctx != NULL);
+	REQUIRE(obj != NULL);
+
+	n = snprintf(buf, sizeof(buf), "%u%%", obj->value.uint32);
+	INSIST(n > 0 && (size_t)n < sizeof(buf));
+	cfg_print_chars(pctx, buf, strlen(buf));
+}
+
+isc_uint32_t
+cfg_obj_aspercentage(const cfg_obj_t *obj) {
+	REQUIRE(obj != NULL && obj->type->rep == &cfg_rep_percentage);
+	return (obj->value.uint32);
+}
+
+LIBISCCFG_EXTERNAL_DATA cfg_type_t cfg_type_percentage = {
+	"percentage", cfg_parse_percentage, cfg_print_percentage,
+	cfg_doc_terminal, &cfg_rep_percentage, NULL
+};
+
+isc_boolean_t
+cfg_obj_ispercentage(const cfg_obj_t *obj) {
+	REQUIRE(obj != NULL);
+	return (ISC_TF(obj->type->rep == &cfg_rep_percentage));
+}
+
 /*
  * Fixed point
  */
@@ -766,6 +873,12 @@ LIBISCCFG_EXTERNAL_DATA cfg_type_t cfg_type_fixedpoint = {
 	cfg_doc_terminal, &cfg_rep_fixedpoint, NULL
 };
 
+isc_boolean_t
+cfg_obj_isfixedpoint(const cfg_obj_t *obj) {
+	REQUIRE(obj != NULL);
+	return (ISC_TF(obj->type->rep == &cfg_rep_fixedpoint));
+}
+
 /*
  * uint32
  */
@@ -967,6 +1080,51 @@ cfg_parse_sstring(cfg_parser_t *pctx, const cfg_type_t *type,
 	return (result);
 }
 
+static isc_result_t
+parse_btext(cfg_parser_t *pctx, const cfg_type_t *type,
+	    cfg_obj_t **ret)
+{
+	isc_result_t result;
+
+	UNUSED(type);
+
+	CHECK(cfg_gettoken(pctx, ISC_LEXOPT_BTEXT));
+	if (pctx->token.type != isc_tokentype_btext) {
+		cfg_parser_error(pctx, CFG_LOG_NEAR,
+				 "expected bracketed text");
+		return (ISC_R_UNEXPECTEDTOKEN);
+	}
+	return (create_string(pctx,
+			      TOKEN_STRING(pctx),
+			      &cfg_type_bracketed_text,
+			      ret));
+ cleanup:
+	return (result);
+}
+
+static void
+print_btext(cfg_printer_t *pctx, const cfg_obj_t *obj) {
+	/*
+	 * We need to print "{" instead of running print_open()
+	 * in order to preserve the exact original formatting
+	 * of the bracketed text. But we increment the indent value
+	 * so that print_close() will leave us back in our original
+	 * state.
+	 */
+	pctx->indent++;
+	cfg_print_cstr(pctx, "{");
+	cfg_print_chars(pctx, obj->value.string.base, obj->value.string.length);
+	print_close(pctx);
+}
+
+static void
+doc_btext(cfg_printer_t *pctx, const cfg_type_t *type) {
+	UNUSED(type);
+
+	cfg_print_cstr(pctx, "{ <unspecified-text> }");
+}
+
+
 isc_boolean_t
 cfg_is_enum(const char *s, const char *const *enums) {
 	const char * const *p;
@@ -1012,16 +1170,17 @@ cfg_parse_enum(cfg_parser_t *pctx, const cfg_type_t *type, cfg_obj_t **ret) {
 void
 cfg_doc_enum(cfg_printer_t *pctx, const cfg_type_t *type) {
 	const char * const *p;
+
 	REQUIRE(pctx != NULL);
 	REQUIRE(type != NULL);
 
-	cfg_print_chars(pctx, "( ", 2);
+	cfg_print_cstr(pctx, "( ");
 	for (p = type->of; *p != NULL; p++) {
 		cfg_print_cstr(pctx, *p);
 		if (p[1] != NULL)
-			cfg_print_chars(pctx, " | ", 3);
+			cfg_print_cstr(pctx, " | ");
 	}
-	cfg_print_chars(pctx, " )", 2);
+	cfg_print_cstr(pctx, " )");
 }
 
 void
@@ -1034,21 +1193,21 @@ cfg_print_ustring(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 
 static void
 print_qstring(cfg_printer_t *pctx, const cfg_obj_t *obj) {
-	cfg_print_chars(pctx, "\"", 1);
+	cfg_print_cstr(pctx, "\"");
 	cfg_print_ustring(pctx, obj);
-	cfg_print_chars(pctx, "\"", 1);
+	cfg_print_cstr(pctx, "\"");
 }
 
 static void
 print_sstring(cfg_printer_t *pctx, const cfg_obj_t *obj) {
-	cfg_print_chars(pctx, "\"", 1);
+	cfg_print_cstr(pctx, "\"");
 	if ((pctx->flags & CFG_PRINTER_XKEY) != 0) {
 		unsigned int len = obj->value.string.length;
 		while (len-- > 0)
-			cfg_print_chars(pctx, "?", 1);
+			cfg_print_cstr(pctx, "?");
 	} else
 		cfg_print_ustring(pctx, obj);
-	cfg_print_chars(pctx, "\"", 1);
+	cfg_print_cstr(pctx, "\"");
 }
 
 static void
@@ -1096,6 +1255,16 @@ LIBISCCFG_EXTERNAL_DATA cfg_type_t cfg_type_sstring = {
 	&cfg_rep_string, NULL
 };
 
+/*
+ * Text enclosed in brackets. Used to pass a block of configuration
+ * text to dynamic library or external application. Checked for
+ * bracket balance, but not otherwise parsed.
+ */
+LIBISCCFG_EXTERNAL_DATA cfg_type_t cfg_type_bracketed_text = {
+	"bracketed_text", parse_btext, print_btext, doc_btext,
+	&cfg_rep_string, NULL
+};
+
 /*
  * Booleans
  */
@@ -1120,7 +1289,7 @@ cfg_parse_boolean(cfg_parser_t *pctx, const cfg_type_t *type, cfg_obj_t **ret)
 	cfg_obj_t *obj = NULL;
 
 	REQUIRE(pctx != NULL);
-	REQUIRE(ret != NULL && *ret == NULL);
+	REQUIRE(ret != NULL && ret != NULL);
 
 	UNUSED(type);
 
@@ -1162,9 +1331,9 @@ cfg_print_boolean(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 	REQUIRE(obj != NULL);
 
 	if (obj->value.boolean)
-		cfg_print_chars(pctx, "yes", 3);
+		cfg_print_cstr(pctx, "yes");
 	else
-		cfg_print_chars(pctx, "no", 2);
+		cfg_print_cstr(pctx, "no");
 }
 
 LIBISCCFG_EXTERNAL_DATA cfg_type_t cfg_type_boolean = {
@@ -1204,8 +1373,9 @@ create_listelt(cfg_parser_t *pctx, cfg_listelt_t **eltp) {
 }
 
 static void
-free_list_elt(cfg_parser_t *pctx, cfg_listelt_t *elt) {
-	cfg_obj_destroy(pctx, &elt->obj);
+free_listelt(cfg_parser_t *pctx, cfg_listelt_t *elt) {
+	if (elt->obj != NULL)
+		cfg_obj_destroy(pctx, &elt->obj);
 	isc_mem_put(pctx->mctx, elt, sizeof(*elt));
 }
 
@@ -1217,7 +1387,7 @@ free_list(cfg_parser_t *pctx, cfg_obj_t *obj) {
 	     elt = next)
 	{
 		next = ISC_LIST_NEXT(elt, link);
-		free_list_elt(pctx, elt);
+		free_listelt(pctx, elt);
 	}
 }
 
@@ -1278,7 +1448,7 @@ parse_list(cfg_parser_t *pctx, const cfg_type_t *listtype, cfg_obj_t **ret)
 
  cleanup:
 	if (elt != NULL)
-		free_list_elt(pctx, elt);
+		free_listelt(pctx, elt);
 	CLEANUP_OBJ(listobj);
 	return (result);
 }
@@ -1290,10 +1460,16 @@ print_list(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 
 	for (elt = ISC_LIST_HEAD(*list);
 	     elt != NULL;
-	     elt = ISC_LIST_NEXT(elt, link)) {
-		print_indent(pctx);
-		cfg_print_obj(pctx, elt->obj);
-		cfg_print_chars(pctx, ";\n", 2);
+	     elt = ISC_LIST_NEXT(elt, link))
+	{
+		if ((pctx->flags & CFG_PRINTER_ONELINE) != 0) {
+			cfg_print_obj(pctx, elt->obj);
+			cfg_print_cstr(pctx, "; ");
+		} else {
+			print_indent(pctx);
+			cfg_print_obj(pctx, elt->obj);
+			cfg_print_cstr(pctx, ";\n");
+		}
 	}
 }
 
@@ -1329,9 +1505,9 @@ cfg_doc_bracketed_list(cfg_printer_t *pctx, const cfg_type_t *type) {
 	REQUIRE(pctx != NULL);
 	REQUIRE(type != NULL);
 
-	cfg_print_chars(pctx, "{ ", 2);
+	cfg_print_cstr(pctx, "{ ");
 	cfg_doc_obj(pctx, type->of);
-	cfg_print_chars(pctx, "; ... }", 7);
+	cfg_print_cstr(pctx, "; ... }");
 }
 
 /*
@@ -1384,7 +1560,7 @@ cfg_print_spacelist(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 	     elt = ISC_LIST_NEXT(elt, link)) {
 		cfg_print_obj(pctx, elt->obj);
 		if (ISC_LIST_NEXT(elt, link) != NULL)
-			cfg_print_chars(pctx, " ", 1);
+			cfg_print_cstr(pctx, " ");
 	}
 }
 
@@ -1517,14 +1693,12 @@ cfg_parse_mapbody(cfg_parser_t *pctx, const cfg_type_t *type, cfg_obj_t **ret)
 		}
 	done:
 		if (clause == NULL || clause->name == NULL) {
-			cfg_parser_error(pctx, CFG_LOG_NOPREP,
-					 "unknown option");
+			cfg_parser_error(pctx, CFG_LOG_NOPREP, "unknown option");
 			/*
 			 * Try to recover by parsing this option as an unknown
 			 * option and discarding it.
 			 */
-			CHECK(cfg_parse_obj(pctx, &cfg_type_unsupported,
-					    &eltobj));
+			CHECK(cfg_parse_obj(pctx, &cfg_type_unsupported, &eltobj));
 			cfg_obj_destroy(pctx, &eltobj);
 			CHECK(parse_semicolon(pctx));
 			continue;
@@ -1533,24 +1707,15 @@ cfg_parse_mapbody(cfg_parser_t *pctx, const cfg_type_t *type, cfg_obj_t **ret)
 		/* Clause is known. */
 
 		/* Issue warnings if appropriate */
-		if ((pctx->flags & CFG_PCTX_NODEPRECATED) == 0 &&
-		    (clause->flags & CFG_CLAUSEFLAG_DEPRECATED) != 0)
-		{
-			cfg_parser_warning(pctx, 0, "option '%s' is deprecated",
-					   clause->name);
-		}
-		if ((clause->flags & CFG_CLAUSEFLAG_OBSOLETE) != 0) {
+		if ((clause->flags & CFG_CLAUSEFLAG_OBSOLETE) != 0)
 			cfg_parser_warning(pctx, 0, "option '%s' is obsolete",
-					   clause->name);
-		}
-		if ((clause->flags & CFG_CLAUSEFLAG_NOTIMP) != 0) {
+				       clause->name);
+		if ((clause->flags & CFG_CLAUSEFLAG_NOTIMP) != 0)
 			cfg_parser_warning(pctx, 0, "option '%s' is "
-					   "not implemented", clause->name);
-		}
-		if ((clause->flags & CFG_CLAUSEFLAG_NYI) != 0) {
+				       "not implemented", clause->name);
+		if ((clause->flags & CFG_CLAUSEFLAG_NYI) != 0)
 			cfg_parser_warning(pctx, 0, "option '%s' is "
-					   "not implemented", clause->name);
-		}
+				       "not implemented", clause->name);
 
 		if ((clause->flags & CFG_CLAUSEFLAG_NOTCONFIGURED) != 0) {
 			cfg_parser_warning(pctx, 0, "option '%s' was not "
@@ -1734,10 +1899,24 @@ cfg_parse_netprefix_map(cfg_parser_t *pctx, const cfg_type_t *type, cfg_obj_t **
 	return (parse_any_named_map(pctx, &cfg_type_netprefix, type, ret));
 }
 
+static void
+print_symval(cfg_printer_t *pctx, const char *name, cfg_obj_t *obj) {
+	if ((pctx->flags & CFG_PRINTER_ONELINE) == 0)
+		print_indent(pctx);
+
+	cfg_print_cstr(pctx, name);
+	cfg_print_cstr(pctx, " ");
+	cfg_print_obj(pctx, obj);
+
+	if ((pctx->flags & CFG_PRINTER_ONELINE) == 0)
+		cfg_print_cstr(pctx, ";\n");
+	else
+		cfg_print_cstr(pctx, "; ");
+}
+
 void
 cfg_print_mapbody(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 	isc_result_t result = ISC_R_SUCCESS;
-
 	const cfg_clausedef_t * const *clauseset;
 
 	REQUIRE(pctx != NULL);
@@ -1764,19 +1943,14 @@ cfg_print_mapbody(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 					for (elt = ISC_LIST_HEAD(*list);
 					     elt != NULL;
 					     elt = ISC_LIST_NEXT(elt, link)) {
-						print_indent(pctx);
-						cfg_print_cstr(pctx, clause->name);
-						cfg_print_chars(pctx, " ", 1);
-						cfg_print_obj(pctx, elt->obj);
-						cfg_print_chars(pctx, ";\n", 2);
+						print_symval(pctx,
+							     clause->name,
+							     elt->obj);
 					}
 				} else {
 					/* Single-valued. */
-					print_indent(pctx);
-					cfg_print_cstr(pctx, clause->name);
-					cfg_print_chars(pctx, " ", 1);
-					cfg_print_obj(pctx, symobj);
-					cfg_print_chars(pctx, ";\n", 2);
+					print_symval(pctx, clause->name,
+						     symobj);
 				}
 			} else if (result == ISC_R_NOTFOUND) {
 				; /* do nothing */
@@ -1831,7 +2005,7 @@ cfg_doc_mapbody(cfg_printer_t *pctx, const cfg_type_t *type) {
 		     clause->name != NULL;
 		     clause++) {
 			cfg_print_cstr(pctx, clause->name);
-			cfg_print_chars(pctx, " ", 1);
+			cfg_print_cstr(pctx, " ");
 			cfg_doc_obj(pctx, clause->type);
 			cfg_print_cstr(pctx, ";");
 			print_clause_flags(pctx, clause->flags);
@@ -1847,7 +2021,7 @@ cfg_print_map(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 
 	if (obj->value.map.id != NULL) {
 		cfg_print_obj(pctx, obj->value.map.id);
-		cfg_print_chars(pctx, " ", 1);
+		cfg_print_cstr(pctx, " ");
 	}
 	print_open(pctx);
 	cfg_print_mapbody(pctx, obj);
@@ -1864,13 +2038,13 @@ cfg_doc_map(cfg_printer_t *pctx, const cfg_type_t *type) {
 
 	if (type->parse == cfg_parse_named_map) {
 		cfg_doc_obj(pctx, &cfg_type_astring);
-		cfg_print_chars(pctx, " ", 1);
+		cfg_print_cstr(pctx, " ");
 	} else if (type->parse == cfg_parse_addressed_map) {
 		cfg_doc_obj(pctx, &cfg_type_netaddr);
-		cfg_print_chars(pctx, " ", 1);
+		cfg_print_cstr(pctx, " ");
 	} else if (type->parse == cfg_parse_netprefix_map) {
 		cfg_doc_obj(pctx, &cfg_type_netprefix);
-		cfg_print_chars(pctx, " ", 1);
+		cfg_print_cstr(pctx, " ");
 	}
 
 	print_open(pctx);
@@ -1882,11 +2056,11 @@ cfg_doc_map(cfg_printer_t *pctx, const cfg_type_t *type) {
 			print_indent(pctx);
 			cfg_print_cstr(pctx, clause->name);
 			if (clause->type->print != cfg_print_void)
-				cfg_print_chars(pctx, " ", 1);
+				cfg_print_cstr(pctx, " ");
 			cfg_doc_obj(pctx, clause->type);
-			cfg_print_chars(pctx, ";", 1);
+			cfg_print_cstr(pctx, ";");
 			print_clause_flags(pctx, clause->flags);
-			cfg_print_chars(pctx, "\n", 1);
+			cfg_print_cstr(pctx, "\n");
 		}
 	}
 	print_close(pctx);
@@ -2059,25 +2233,27 @@ token_addr(cfg_parser_t *pctx, unsigned int flags, isc_netaddr_t *na) {
 				return (ISC_R_SUCCESS);
 			}
 		}
-		if ((flags & CFG_ADDR_V4PREFIXOK) != 0 && strlen(s) <= 15U) {
+		if ((flags & CFG_ADDR_V4PREFIXOK) != 0 &&
+		    strlen(s) <= 15U) {
 			char buf[64];
 			int i;
 
-			strlcpy(buf, s, sizeof(buf));
+			strcpy(buf, s);
 			for (i = 0; i < 3; i++) {
-				strlcat(buf, ".0", sizeof(buf));
+				strcat(buf, ".0");
 				if (inet_pton(AF_INET, buf, &in4a) == 1) {
 					isc_netaddr_fromin(na, &in4a);
 					return (ISC_R_SUCCESS);
 				}
 			}
 		}
-		if ((flags & CFG_ADDR_V6OK) != 0 && strlen(s) <= 127U) {
+		if ((flags & CFG_ADDR_V6OK) != 0 &&
+		    strlen(s) <= 127U) {
 			char buf[128]; /* see lib/bind9/getaddresses.c */
 			char *d; /* zone delimiter */
 			isc_uint32_t zone = 0; /* scope zone ID */
 
-			strlcpy(buf, s, sizeof(buf));
+			strcpy(buf, s);
 			d = strchr(buf, '%');
 			if (d != NULL)
 				*d = '\0';
@@ -2194,7 +2370,8 @@ cfg_print_rawaddr(cfg_printer_t *pctx, const isc_netaddr_t *na) {
 	isc_buffer_init(&buf, text, sizeof(text));
 	result = isc_netaddr_totext(na, &buf);
 	RUNTIME_CHECK(result == ISC_R_SUCCESS);
-	cfg_print_chars(pctx, isc_buffer_base(&buf), isc_buffer_usedlength(&buf));
+	cfg_print_chars(pctx, isc_buffer_base(&buf),
+			isc_buffer_usedlength(&buf));
 }
 
 isc_result_t
@@ -2252,26 +2429,26 @@ cfg_doc_netaddr(cfg_printer_t *pctx, const cfg_type_t *type) {
 	const unsigned int *flagp = type->of;
 	int n = 0;
 	if (*flagp != CFG_ADDR_V4OK && *flagp != CFG_ADDR_V6OK)
-		cfg_print_chars(pctx, "( ", 2);
+		cfg_print_cstr(pctx, "( ");
 	if (*flagp & CFG_ADDR_V4OK) {
 		cfg_print_cstr(pctx, "<ipv4_address>");
 		n++;
 	}
 	if (*flagp & CFG_ADDR_V6OK) {
 		if (n != 0)
-			cfg_print_chars(pctx, " | ", 3);
+			cfg_print_cstr(pctx, " | ");
 		cfg_print_cstr(pctx, "<ipv6_address>");
 		n++;
 	}
 	if (*flagp & CFG_ADDR_WILDOK) {
 		if (n != 0)
-			cfg_print_chars(pctx, " | ", 3);
-		cfg_print_chars(pctx, "*", 1);
+			cfg_print_cstr(pctx, " | ");
+		cfg_print_cstr(pctx, "*");
 		n++;
 		POST(n);
 	}
 	if (*flagp != CFG_ADDR_V4OK && *flagp != CFG_ADDR_V6OK)
-		cfg_print_chars(pctx, " )", 2);
+		cfg_print_cstr(pctx, " )");
 }
 
 LIBISCCFG_EXTERNAL_DATA cfg_type_t cfg_type_netaddr = {
@@ -2362,7 +2539,7 @@ print_netprefix(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 	const cfg_netprefix_t *p = &obj->value.netprefix;
 
 	cfg_print_rawaddr(pctx, &p->address);
-	cfg_print_chars(pctx, "/", 1);
+	cfg_print_cstr(pctx, "/");
 	cfg_print_rawuint(pctx, p->prefixlen);
 }
 
@@ -2481,11 +2658,11 @@ cfg_print_sockaddr(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 	cfg_print_cstr(pctx, buf);
 	port = isc_sockaddr_getport(&obj->value.sockaddr);
 	if (port != 0) {
-		cfg_print_chars(pctx, " port ", 6);
+		cfg_print_cstr(pctx, " port ");
 		cfg_print_rawuint(pctx, port);
 	}
 	if (obj->value.sockaddrdscp.dscp != -1) {
-		cfg_print_chars(pctx, " dscp ", 6);
+		cfg_print_cstr(pctx, " dscp ");
 		cfg_print_rawuint(pctx, obj->value.sockaddrdscp.dscp);
 	}
 }
@@ -2498,25 +2675,25 @@ cfg_doc_sockaddr(cfg_printer_t *pctx, const cfg_type_t *type) {
 	REQUIRE(pctx != NULL);
 	REQUIRE(type != NULL);
 
-	cfg_print_chars(pctx, "( ", 2);
+	cfg_print_cstr(pctx, "( ");
 	if (*flagp & CFG_ADDR_V4OK) {
 		cfg_print_cstr(pctx, "<ipv4_address>");
 		n++;
 	}
 	if (*flagp & CFG_ADDR_V6OK) {
 		if (n != 0)
-			cfg_print_chars(pctx, " | ", 3);
+			cfg_print_cstr(pctx, " | ");
 		cfg_print_cstr(pctx, "<ipv6_address>");
 		n++;
 	}
 	if (*flagp & CFG_ADDR_WILDOK) {
 		if (n != 0)
-			cfg_print_chars(pctx, " | ", 3);
-		cfg_print_chars(pctx, "*", 1);
+			cfg_print_cstr(pctx, " | ");
+		cfg_print_cstr(pctx, "*");
 		n++;
 		POST(n);
 	}
-	cfg_print_chars(pctx, " ) ", 3);
+	cfg_print_cstr(pctx, " ) ");
 	if (*flagp & CFG_ADDR_WILDOK) {
 		cfg_print_cstr(pctx, "[ port ( <integer> | * ) ]");
 	} else {
@@ -2725,13 +2902,14 @@ parser_complain(cfg_parser_t *pctx, isc_boolean_t is_warning,
 	if (have_current_file(pctx))
 		snprintf(where, sizeof(where), "%s:%u: ",
 			 current_file(pctx), pctx->line);
+	else if (pctx->buf_name != NULL)
+		snprintf(where, sizeof(where), "%s: ", pctx->buf_name);
 
 	len = vsnprintf(message, sizeof(message), format, args);
 #define ELIPSIS " ... "
-	if (len >= sizeof(message)) {
-		message[sizeof(message) - sizeof(ELIPSIS)] = 0;
-		strlcat(message, ELIPSIS, sizeof(message));
-	}
+	if (len >= sizeof(message))
+		strcpy(message + sizeof(message) - sizeof(ELIPSIS) - 1,
+		       ELIPSIS);
 
 	if ((flags & (CFG_LOG_NEAR|CFG_LOG_BEFORE|CFG_LOG_NOPREP)) != 0) {
 		isc_region_t r;
@@ -2782,9 +2960,9 @@ cfg_obj_log(const cfg_obj_t *obj, isc_log_t *lctx, int level,
 		return;
 
 	va_start(ap, fmt);
-
 	vsnprintf(msgbuf, sizeof(msgbuf), fmt, ap);
 	va_end(ap);
+
 	if (obj->file != NULL) {
 		isc_log_write(lctx, CAT, MOD, level,
 			      "%s:%u: %s", obj->file, obj->line, msgbuf);
@@ -2819,15 +2997,19 @@ cfg_create_obj(cfg_parser_t *pctx, const cfg_type_t *type, cfg_obj_t **ret) {
 	obj = isc_mem_get(pctx->mctx, sizeof(cfg_obj_t));
 	if (obj == NULL)
 		return (ISC_R_NOMEMORY);
+
 	obj->type = type;
 	obj->file = current_file(pctx);
 	obj->line = pctx->line;
+	obj->pctx = pctx;
+
 	result = isc_refcount_init(&obj->references, 1);
 	if (result != ISC_R_SUCCESS) {
 		isc_mem_put(pctx->mctx, obj, sizeof(cfg_obj_t));
 		return (result);
 	}
 	*ret = obj;
+
 	return (ISC_R_SUCCESS);
 }
 
@@ -2845,7 +3027,6 @@ map_symtabitem_destroy(char *key, unsigned int type,
 	cfg_obj_destroy(pctx, &obj);
 }
 
-
 static isc_result_t
 create_map(cfg_parser_t *pctx, const cfg_type_t *type, cfg_obj_t **ret) {
 	isc_result_t result;
@@ -2933,9 +3114,9 @@ cfg_doc_terminal(cfg_printer_t *pctx, const cfg_type_t *type) {
 	REQUIRE(pctx != NULL);
 	REQUIRE(type != NULL);
 
-	cfg_print_chars(pctx, "<", 1);
+	cfg_print_cstr(pctx, "<");
 	cfg_print_cstr(pctx, type->name);
-	cfg_print_chars(pctx, ">", 1);
+	cfg_print_cstr(pctx, ">");
 }
 
 void
@@ -2951,3 +3132,73 @@ cfg_print_grammar(const cfg_type_t *type,
 	pctx.flags = 0;
 	cfg_doc_obj(&pctx, type);
 }
+
+isc_result_t
+cfg_parser_mapadd(cfg_parser_t *pctx, cfg_obj_t *mapobj,
+		  cfg_obj_t *obj, const char *clausename)
+{
+	isc_result_t result = ISC_R_SUCCESS;
+	const cfg_map_t *map;
+	isc_symvalue_t symval;
+	cfg_obj_t *destobj = NULL;
+	cfg_listelt_t *elt = NULL;
+	const cfg_clausedef_t * const *clauseset;
+	const cfg_clausedef_t *clause;
+
+	REQUIRE(pctx != NULL);
+	REQUIRE(mapobj != NULL && mapobj->type->rep == &cfg_rep_map);
+	REQUIRE(obj != NULL);
+	REQUIRE(clausename != NULL);
+
+	map = &mapobj->value.map;
+
+	clause = NULL;
+	for (clauseset = map->clausesets; *clauseset != NULL; clauseset++) {
+		for (clause = *clauseset; clause->name != NULL; clause++) {
+			if (strcasecmp(clause->name, clausename) == 0) {
+				goto breakout;
+			}
+		}
+	}
+
+ breakout:
+	if (clause == NULL || clause->name == NULL)
+		return (ISC_R_FAILURE);
+
+	result = isc_symtab_lookup(map->symtab, clausename, 0, &symval);
+	if (result == ISC_R_NOTFOUND) {
+		if ((clause->flags & CFG_CLAUSEFLAG_MULTI) != 0) {
+			CHECK(cfg_create_list(pctx, &cfg_type_implicitlist,
+					      &destobj));
+			CHECK(create_listelt(pctx, &elt));
+			cfg_obj_attach(obj, &elt->obj);
+			ISC_LIST_APPEND(destobj->value.list, elt, link);
+			symval.as_pointer = destobj;
+		} else
+			symval.as_pointer = obj;
+
+		CHECK(isc_symtab_define(map->symtab, clausename, 1, symval,
+					isc_symexists_reject));
+	} else {
+		cfg_obj_t *destobj2 = symval.as_pointer;
+
+		INSIST(result == ISC_R_SUCCESS);
+
+		if (destobj2->type == &cfg_type_implicitlist) {
+			CHECK(create_listelt(pctx, &elt));
+			cfg_obj_attach(obj, &elt->obj);
+			ISC_LIST_APPEND(destobj2->value.list, elt, link);
+		} else
+			result = ISC_R_EXISTS;
+	}
+
+	destobj = NULL;
+	elt = NULL;
+
+ cleanup:
+	if (elt != NULL)
+		free_listelt(pctx, elt);
+	CLEANUP_OBJ(destobj);
+
+	return (result);
+}
